module Manatee.Data.HsInstance where

import System.IO
import System.Exit
import System.Environment
import System.Console.GetOpt
import Language.Haskell.Exts.Syntax
import Language.Haskell.Exts.Fixity
import Language.Haskell.Exts.Pretty
import Language.Haskell.Exts.Extension
import Language.Haskell.Exts.Parser(ParseMode(..),ParseResult(..))
import Language.Haskell.Meta(parseResultToEither,moduleDecls)

import qualified Language.Haskell.Exts.Parser as P
import qualified Language.Haskell.Exts as P

hsInstance :: FilePath -> IO [(Context, QName, [Type])]
hsInstance filepath = do
  content <- readFile filepath
  return $ case pipeline content of
             Left _  -> error "invalid content to parse" 
             Right r -> r

hsInstanceString :: FilePath -> IO [(String, String)]
hsInstanceString filepath = do
  info <- hsInstance filepath
  return $ map (\(_,UnQual (Ident className), [TyCon (UnQual (Ident typeName))]) -> (className, typeName)) info 

parseHsModule :: String -> Either String Module
parseHsModule = parse P.parseFileContentsWithMode

pipeline :: String -> Either String [(Context,QName,[Type])]
pipeline = either Left
           (Right . getInstanceInfo . moduleDecls) . parseHsModule

getInstanceInfo :: [Decl] -> [(Context,QName,[Type])]
getInstanceInfo decls = [(cxt,qname,tys)
                          | InstDecl loc cxt qname tys idecls <- decls]

parse :: (ParseMode -> String -> ParseResult a) -> String -> Either String a
parse p = parseResultToEither . p hsInstanceParseMode

hsInstanceParseMode :: ParseMode
hsInstanceParseMode = ParseMode
  {parseFilename = []
  ,extensions = hsInstanceExtensions
  ,ignoreLanguagePragmas = False
  ,fixities = baseFixities}

hsInstanceExtensions :: [Extension]
hsInstanceExtensions =
  [OverlappingInstances
  ,UndecidableInstances
  ,IncoherentInstances
  ,RecursiveDo
  ,ParallelListComp
  ,MultiParamTypeClasses
  ,NoMonomorphismRestriction
  ,FunctionalDependencies
  ,RankNTypes
  ,PolymorphicComponents
  ,ExistentialQuantification
  ,ScopedTypeVariables
  ,ImplicitParams
  ,FlexibleContexts
  ,FlexibleInstances
  ,EmptyDataDecls
  ,CPP
  ,KindSignatures
  ,BangPatterns
  ,TypeSynonymInstances
  ,TemplateHaskell
  ,ForeignFunctionInterface
  ,NamedFieldPuns
  ,PatternGuards
  ,GeneralizedNewtypeDeriving
  ,ExtensibleRecords
  ,HereDocuments
  ,MagicHash
  ,TypeFamilies
  ,StandaloneDeriving
  ,UnicodeSyntax
  ,PatternSignatures
  ,UnliftedFFITypes
  ,LiberalTypeSynonyms
  ,TypeOperators
  ,RecordWildCards
  ,RecordPuns
  ,DisambiguateRecordFields
  ,OverloadedStrings
  ,GADTs
  ,RelaxedPolyRec
  ,ExtendedDefaultRules
  ,UnboxedTuples
  ,DeriveDataTypeable
  ,ConstrainedClassMethods
  ,PackageImports
  ,ImpredicativeTypes
  ,NewQualifiedOperators
  ,PostfixOperators
  ,QuasiQuotes
  ,TransformListComp
  ,ViewPatterns]
