{-# LANGUAGE ExistentialQuantification, RankNTypes #-}
module Manatee.Keymap.Keymap where

import Data.Map (Map)

import Manatee.IORefObject
import Manatee.UI.Page
import Manatee.Event.Event
import Manatee.Utils.Generic
import {-# source #-} Manatee.PageMode.PageMode
import Manatee.PageView.PageView

import Manatee.Keymap.View
import Manatee.Keymap.Page
import Manatee.Keymap.TabGroup
import Manatee.Keymap.Misc
import Manatee.Keymap.Interactivebar

import qualified Data.Map as M
import qualified Graphics.UI.Gtk.Gdk.Events as E

data Action = forall e . ActionExecute e => Action e

class ActionExecute e where
    execute     :: e -> IORefObject -> IO ()

type Keymap = Map String Action

-- | Global keymap
globalKeymap :: Keymap
globalKeymap = 
    M.unions [viewKeymap
             ,viewListKeymap
             ,tabGroupKeymap
             ,pageKeymap
             ,pageViewKeymap
             ,miscKeymap]

-- | Main key press handler.
mainKeyPressHandler :: E.Event -> IORefObject -> IO Bool
mainKeyPressHandler ev ioRefObject = 
  case eventTransform ev of
    Nothing -> return False
    Just e -> do
      let eventName = eventGetName e
          handler keymap = keymapLookup keymap eventName ioRefObject
      page <- ioRefObjectGetFocusPage ioRefObject
      case page of
        Just p  -> do
                  let mKeymap = pageModeApply modeKeymap $ pageMode p
                      bKeymap = pageApplyBuffer pageViewGetKeymap p
                  focusInteractivebar <- pageViewIsFocusInteractivebar $ pageApplyBuffer pageViewUIFrame p
                  if focusInteractivebar 
                     -- When interactivebar is focus.
                     then 
                         -- Lookup interactivar keymap.
                         ifM (handler interactivebarKeymap)
                                 (return True)
                                 -- Lookup global keymap
                                 (handler globalKeymap)
                     else 
                         -- First lookup local mode keymap.
                         ifM (handler mKeymap)
                                 (return True)
                                 -- Second lookup local buffer keymap. 
                                 (ifM (handler bKeymap)
                                          (return True)
                                          -- Last lookup global keymap.
                                          (handler globalKeymap))
        Nothing -> handler globalKeymap

-- | Lookup keymap.
keymapLookup :: Keymap -> String -> IORefObject -> IO Bool
keymapLookup keymap key ioRefObject = 
    case M.lookup key keymap of
      Just (Action e) -> execute e ioRefObject >> return True
      Nothing -> return False

-- | Create empty keymap.
keymapEmpty :: Keymap
keymapEmpty = M.fromList []

