{-# LANGUAGE ExistentialQuantification, DeriveDataTypeable, RankNTypes #-}
module Manatee.PageMode.PageMode where

import Data.Typeable
import Text.Regex.TDFA
import Data.List

import Manatee.UI.Page
import Manatee.Keymap.Keymap

import qualified Manatee.PageMode.C         as C
import qualified Manatee.PageMode.Haskell   as Haskell

data PageMode = forall syntax . PageMode (Mode syntax)
               deriving Typeable

data Mode syntax = 
    Mode {modeName      :: String -- mode name
         ,modeRegexp    :: String -- regular expression for match file
         ,modeKeymap    :: Keymap -- local keymap
         ,modeLoad      :: Page -> IO () -- local mode load
         }

-- | Default empty mode for init.
emptyMode :: Mode syntax
emptyMode = 
    Mode {modeName = "empty"
         ,modeRegexp = ""
         ,modeKeymap = keymapEmpty
         ,modeLoad = (\_ -> return ())}

-- | Page mode table for lookup.
pageModeTable :: [PageMode]
pageModeTable = 
    [PageMode C.cMode
    ,PageMode Haskell.haskellMode]

-- | Lookup page mode with filepath.
pageModeLookup :: String -> PageMode
pageModeLookup filepath = 
  case mode of
    -- Return corresponding mode if found match.
    Just m  -> m
    -- Otherwise return empty mode.
    Nothing -> PageMode emptyMode
  where
    -- Find mode that regexp match.
    mode = find (\x -> filepath =~ pageModeApply modeRegexp x :: Bool) pageModeTable

-- | Apply function with mode.
pageModeApply :: (forall syntax . Mode syntax -> b) -> PageMode -> b
pageModeApply f (PageMode m) = f m

-- | Load local mode.
pageModeLoad :: PageMode -> Page -> IO ()
pageModeLoad (PageMode m) = modeLoad m
