{-# LANGUAGE DeriveDataTypeable, TemplateHaskell #-}
module Manatee.State.Page where

import Data.DeriveTH
import Data.Set (Set)
import Data.Binary
import Data.Sequence (Seq)
import Data.IORef
import Data.Function
import Data.Ord

import Manatee.PageView.PageView
import Manatee.IORefObject
import Manatee.Data.SetList
import Manatee.Utils.Generic
import Manatee.UI.TabGroup
import Manatee.UI.Page
import Manatee.UI.View

import qualified Manatee.Utils.Set as MUS 
import qualified Data.Set as Set
import qualified Data.Foldable as F

data PageState =
    PageState {psId     :: Int
              ,psWrap   :: PageViewStateWrap}

data PageListState =
    PageListState {plsSet       :: Set PageState
                  ,plsCounter   :: Int
                  ,plsSequence  :: Maybe (Seq Int)}

instance Eq PageState where
    (==) = (==) `on` psId

instance Ord PageState where
    compare = comparing psId

-- | Get page state.
pageStateGet :: Page -> IO PageState    
pageStateGet p = do
  let pId = pageId p
  wrap <- pageApplyBuffer pageViewGetState p
  return $ PageState pId wrap
  
-- | Get page list state.
pageListStateGet :: IORefObject -> IO PageListState
pageListStateGet ioRefObject = do
  -- Get init value.
  pageList <- readIORef $ ioRefObjectPageList ioRefObject
  let pageCounter = listCounter pageList
      pageSet     = listSet pageList

  -- Get page state.
  set <- MUS.mapM pageStateGet pageSet

  -- Get page sequence.
  seq <- applyMaybeM (readIORef . tabGroupPageIdSequence . viewTabGroup
                    ) =<< ioRefObjectGetFocusView ioRefObject 

  return $ PageListState set pageCounter seq

-- | Rebuild page state.
pageStateRebuild :: PageState -> IORefObject -> IO ()  
pageStateRebuild (PageState pId (PageViewStateWrap pvs)) =
    rebuild pvs pId

-- | Rebuld page list state.
pageListStateRebuild :: PageListState -> IORefObject -> IO ()
pageListStateRebuild state ioRefObject = do
  -- Get init value.
  let pageList = ioRefObjectPageList ioRefObject
      set      = plsSet state
      counter  = plsCounter state
      sequence = plsSequence state

  -- Set page list counter.
  modifyIORef pageList (\x -> setListSetCounter x counter)

  -- Rebuld pages.
  applyMaybeTM_ (mapM_ (\i -> 
                            applyMaybeTM_ (\x -> pageStateRebuild x ioRefObject
                                          ) $ MUS.maybeFindMin set (\x -> psId x == i)
                       ) . F.toList
                ) sequence
  
-- | Generate binary isntance automatcially.
$(derive makeBinary ''PageState)
$(derive makeBinary ''PageListState)
