{-# LANGUAGE DeriveDataTypeable, TemplateHaskell #-}
module Manatee.State.View where

import Data.DeriveTH
import Data.Binary

import Manatee.Utils.Generic
import Manatee.UI.View
import Manatee.UI.ViewNode
import Manatee.UI.TabGroup
import Manatee.IORefObject

data ViewState =
    ViewState {vsId     :: Int
              ,vsTabId  :: Maybe Int}

data ViewListState =
    ViewListState {vlsFocusViewId       :: Maybe Int
                  ,vlsState             :: [ViewState]}

-- | Get view state.
viewStateGet :: View -> IO ViewState
viewStateGet v = do
  let id = viewNodeId $ viewNode v
  tabId  <- viewGetCurrentPageSequenceId v
  return $ ViewState id tabId 
  
-- | Get view list state.
viewListStateGet :: IORefObject -> IO ViewListState
viewListStateGet ioRefObject = do
  -- Get init value.
  viewList  <- viewListGetList ioRefObject
  focusView <- ioRefObjectGetFocusView ioRefObject

  -- Get focus view id.
  let focusId = applyMaybe (viewNodeId . viewNode) focusView

  -- Get view state.
  viewStateList <- mapM viewStateGet viewList

  return $ ViewListState focusId viewStateList

-- | Rebuild view state.
viewStateRebuild :: ViewState -> IORefObject -> IO ()
viewStateRebuild (ViewState vId vTabId) = 
    viewApplyWithId (\v -> 
                         applyMaybeTM_ (\i -> do
                                          let tabGroup = viewTabGroup v
                                          applyMaybeTM_ (tabGroupFocusPage tabGroup
                                                        ) =<< tabGroupGetPageSequncePage tabGroup i
                                       ) vTabId
                    ) vId
  
-- | Rebuild view list state.
viewListStateRebuild :: ViewListState -> IORefObject -> IO ()
viewListStateRebuild state ioRefObject = do
  -- Get init value.
  let focusViewId = vlsFocusViewId state
      stateList   = vlsState state
      viewList    = ioRefObjectViewList ioRefObject

  -- Rebuild state.
  mapM_ (\x -> viewStateRebuild x ioRefObject) stateList

  -- Focus view.
  applyMaybeTM_ (\i -> viewApplyWithId (viewListFocus viewList) i ioRefObject) focusViewId

-- | Generate binary isntance automatically.
$(derive makeBinary ''ViewState)
$(derive makeBinary ''ViewListState)
