module Manatee.UI.Interactivebar where

import Data.IORef
import Control.Monad

import Graphics.UI.Gtk

import Manatee.Utils.Gtk.Gtk
import Manatee.Utils.Generic
import Manatee.IORefObject
import {-# source #-} Manatee.UI.Page
import {-# source #-} Manatee.PageView.PageView

data Interactivebar =
    Interactivebar {interactivebarFrame                 :: Frame -- frame for decorative
                   ,interactivebarBox                   :: HBox  -- box for container
                   ,interactivebarTitle                 :: Label -- title label
                   ,interactivebarEntry                 :: Entry -- main entry
                   ,interactivebarStatus                :: Label -- status label
                   ,interactivebarAction                :: IORef (Maybe InteractiveAction) -- action
                   ,interactivebarKeyPressSignal        :: IORef (Maybe (ConnectId Entry)) -- key press signal
                   ,interactivebarKeyReleaseSignal      :: IORef (Maybe (ConnectId Entry)) -- key release signal
                   }

type InteractiveAction = String -> IORefObject -> IO ()

-- | Create new interactivebar.
interactivebarNew :: IO Interactivebar
interactivebarNew = do
  -- Create interactivebar.
  frame  <- frameNew
  box    <- hBoxNew False 0
  title  <- labelNew $ Just ""
  entry  <- entryNew
  status <- labelNew $ Just ""
  action <- newIORef Nothing
  pressSignal   <- newIORef Nothing
  releaseSignal <- newIORef Nothing

  -- Connect module.
  frame `containerAdd` box
  boxTryPack box title  PackNatural (Just 0)
  boxTryPack box entry  PackGrow    (Just 1)
  boxTryPack box status PackNatural (Just 2)

  return $ Interactivebar frame box title entry status action pressSignal releaseSignal

-- | Show interactivebar.
interactivebarShow :: String -> (String -> IORefObject -> IO ()) -> IORefObject -> IO ()
interactivebarShow title action ioRefObject = 
    applyMaybeTM_ (\p -> do
                     -- Init.
                     let frame  = pageApplyBuffer pageViewUIFrame p
                         box    = pageViewUIFrameBox frame
                         bar    = pageViewUIFrameInteractivebar frame
                         iFrame = interactivebarFrame bar
                         entry  = interactivebarEntry bar
                         titleLabel    = interactivebarTitle bar
                         statusLabel   = interactivebarStatus bar
                         pressSignal   = interactivebarKeyPressSignal bar 
                         releaseSignal = interactivebarKeyReleaseSignal bar 
                     
                     -- Clean first.
                     applyMaybeTM_ signalDisconnect =<< readIORef pressSignal
                     applyMaybeTM_ signalDisconnect =<< readIORef releaseSignal
                     labelSetText titleLabel ""
                     entrySetText entry ""
                     labelSetText statusLabel ""

                     -- Connect to frame.
                     hasOutputbar <- pageViewIsOutputbarShow frame 
                     let packId = if hasOutputbar then 1 else 0 -- get pack id along with outputbar position
                     boxTryPack box iFrame PackNatural (Just packId)

                     -- Focus entry.
                     widgetShowAll iFrame
                     widgetGrabFocus entry
                     
                     -- Set title
                     labelSetText titleLabel title

                     -- Add action.
                     modifyIORef (interactivebarAction bar) (const $ Just action)
                     
                     -- Add event for entry.
                     ps <- entry `onKeyPress` (\_ -> return False)
                     rs <- entry `onKeyRelease` (\_ -> return False)
                     modifyIORef pressSignal (const $ Just ps)
                     modifyIORef releaseSignal (const $ Just rs)

                     return ()
                   ) =<< ioRefObjectGetFocusPage ioRefObject

-- | Apply action for interactivebar.
interactivebarApplyAction :: (Interactivebar -> IORefObject -> IO ()) -> IORefObject -> IO ()
interactivebarApplyAction action ioRefObject =
    applyMaybeTM_ (\b -> action b ioRefObject
                  ) =<< ioRefObjectGetFocusInteractivebar ioRefObject
    
-- | Exit interactivebar.
interactivebarExit :: Interactivebar -> IORefObject -> IO ()
interactivebarExit _ ioRefObject = 
    applyMaybeTM_ (\p -> do
                     -- Init.
                     let frame  = pageApplyBuffer pageViewUIFrame p
                         box    = pageViewUIFrameBox frame
                         bar    = pageViewUIFrameInteractivebar frame
                         iFrame = interactivebarFrame bar

                     -- Remove from parent.
                     containerTryRemove box iFrame
                  ) =<< ioRefObjectGetFocusPage ioRefObject

-- | Apply candidate.
interactivebarApplyCandidate :: Interactivebar -> IORefObject -> IO ()
interactivebarApplyCandidate bar ioRefObject = do
  -- Exit interactivebar first.
  interactivebarExit bar ioRefObject

  -- Execute action.
  action <- readIORef $ interactivebarAction bar
  result <- entryGetText $ interactivebarEntry bar
  applyMaybeTM_ (\a -> a result ioRefObject) action
