module Manatee.Utils.System where

import Control.Monad
import Data.List
import System.IO

import Manatee.UI.Page
import Manatee.UI.View
import Manatee.Widget.Tooltip
import Manatee.UI.TabGroup
import Manatee.PageView.StringView 
import {-# source #-} Manatee.PageView.DiredView
import Manatee.PageView.TerminalView
import Manatee.PageView.PageView
import Manatee.IORefObject
import Manatee.Utils.Generic
import Manatee.UI.Interactivebar

-- | Open file.
fileOpen :: FilePath -> IORefObject -> IO ()
fileOpen path ioRefObject = 
    applyMaybeTM_ (\view -> do
                     -- Get page list.
                     pageList <- viewGetPageList view

                     -- Get page buffer name list.
                     pageViewNameList <- mapM pageGetBufferName pageList

                     -- File open.
                     case path `elemIndex` pageViewNameList of
                       -- Switch to page if found same one.
                       Just i  -> tabGroupFocusPage (viewTabGroup view) $ pageList !! i
                       -- Otherwise create new page.
                       Nothing -> fileNew path ioRefObject) 
        =<< ioRefObjectGetFocusView ioRefObject

-- | Create new file.
fileNew :: FilePath -> IORefObject -> IO ()
fileNew path ioRefObject = 
    stringViewRebuildState (StringViewState path Nothing) Nothing ioRefObject

fileTemp :: IORefObject -> IO ()
fileTemp ioRefObject = do
  fileOpen "/test/Download/File.hs" ioRefObject
  fileOpen "/test/Download/Test.hs" ioRefObject
  fileOpen "/test/Download/VTree.hs" ioRefObject
  fileOpen "/test/Download/Lang.hs" ioRefObject
  fileOpen "/test/Download/Regex.hs" ioRefObject
  fileOpen "/test/Download/StringLike.hs" ioRefObject
  fileOpen "/test/Download/ViewNode.hs" ioRefObject
  fileOpen "/test/Download/ZList.hs" ioRefObject
  fileOpen "/test/Download/StringBuffer.hs" ioRefObject
  -- fileOpen "/home/andy/temp/File.hs" ioRefObject
  -- fileOpen "/home/andy/temp/PageBuffer.hs" ioRefObject
  -- fileOpen "/home/andy/temp/StringBuffer.hs" ioRefObject
  -- fileOpen "/home/andy/temp/Test.hs" ioRefObject

-- | Open current directory.
dired :: IORefObject -> IO ()
dired ioRefObject = 
    applyMaybeTM_ (\p -> do
                     -- Path.
                     path <- (<<<=) getUpperDirectory (pageApplyBuffer pageViewGetName) p 

                     diredInternal path ioRefObject
                  ) =<< ioRefObjectGetFocusPage ioRefObject

-- | Internal function for dired.
diredInternal :: FilePath -> IORefObject -> IO ()
diredInternal path ioRefObject = 
    applyMaybeTM_ (\view -> do
                     -- Get page list.
                     pageList <- viewGetPageList view

                     -- Get page view name list.
                     pageViewNameList <- mapM pageGetBufferName pageList

                     -- File open.
                     case path `elemIndex` pageViewNameList of
                       -- Switch to page if found same one.
                       Just i  -> tabGroupFocusPage (viewTabGroup view) $ pageList !! i
                       -- Otherwise create new page.
                       Nothing -> diredNew path ioRefObject
                    ) =<< ioRefObjectGetFocusView ioRefObject

-- | Dired new.
diredNew :: FilePath -> IORefObject -> IO ()
diredNew path = diredViewRebuildState path Nothing 

-- | Terminal.
term :: IORefObject -> IO ()
term ioRefObject = do
  name <- terminalGenerateName ioRefObject
  terminalViewRebuildState name Nothing ioRefObject

tooltipTest :: IORefObject -> IO ()
tooltipTest ioRefObject = do
  let rootWindow = ioRefObjectRootWindow ioRefObject
  tooltipNew rootWindow "Tooltip A" 100 100 5000 
  tooltipNew rootWindow "Tooltip B" 300 300 5000 
  return ()
  
findFile :: IORefObject -> IO ()
findFile ioRefObject = interactivebarShow "Find file: " fileOpen ioRefObject
