module Manatee.Widget.Outputbar where

import Data.IORef

import Graphics.UI.Gtk

import Manatee.Utils.Generic
import Manatee.Utils.Gtk.Gtk

data Outputbar =
    Outputbar {outputbarFrame           :: Frame -- frame for decorative
              ,outputbarLabel           :: Label -- label for output
              ,outputbarHandlerId       :: IORef (Maybe HandlerId) -- handlerId for track tiemout
              }

-- | Create outputbar.
outputbarNew :: IO Outputbar
outputbarNew = do
  -- Create frame.
  frame <- frameNew

  -- Create box.
  box <- hBoxNew False 0

  -- Create label.
  label <- labelNew $ Just ""

  -- Create id.
  id <- newIORef Nothing

  -- Connect module.
  frame `containerAdd` box
  boxPackStart box label PackNatural 0

  return $ Outputbar frame label id
  
-- | Show outputbar.  
outputbarShow :: BoxClass box => box -> Outputbar -> String -> IO ()
outputbarShow box bar str = do
  -- Init.
  let frame = outputbarFrame bar
      label = outputbarLabel bar
      handlerId = outputbarHandlerId bar
  id <- readIORef handlerId

  -- Remove previous tiemout handler first.
  applyMaybeTM_ timeoutRemove id
  
  -- Set output.
  labelSetText label str

  -- Connect module.
  boxTryPack box frame PackNatural (Just 0)
  
  -- Show
  widgetShowAll frame
  
  -- Hide outputbar 10s later.
  newId <- timeoutAdd (box `containerRemove` frame >> return False) 10000
  modifyIORef handlerId (const $ Just newId)
