module Manatee.Widget.Statusbar where

import Control.Monad
import Data.IORef
import Data.Map (Map)

import Graphics.UI.Gtk hiding (Statusbar)

import Manatee.Utils.Generic

import qualified Graphics.UI.Gtk.Display.Statusbar as S
import qualified Data.Map as M

type StatusbarSubitem = S.Statusbar
type InfoTable = Map String String
type StatusbarInfoTable = IORef InfoTable

data Statusbar =
    Statusbar {statusbarInfoSubitem     :: StatusbarSubitem
              ,statusbarInfoTable       :: StatusbarInfoTable}

-- | Create new statusbar.
statusbarNew :: BoxClass b => b -> Maybe Statusbar -> IO Statusbar
statusbarNew b statusbar = do
  let box = toBox b

  -- Status box for contain status boxes.
  statusBox <- hBoxNew False 0
  boxPackStart box statusBox PackNatural 0

  infoSubitem   <- statusbarSubitemNew statusBox
  infoTable     <- case statusbar of
                    -- Clone info table.
                    Just s  -> return $ statusbarInfoTable s
                    -- Otherwise new info table.
                    Nothing -> newIORef $ M.fromList []
  return $ Statusbar infoSubitem infoTable

-- | Create new sub-statusbar.
statusbarSubitemNew :: BoxClass b => b -> IO StatusbarSubitem
statusbarSubitemNew b = do
  let box = toBox b

  bar <- S.statusbarNew
  statusbarSetHasResizeGrip bar False -- don't show resize grip
  boxPackStart box bar PackGrow 0     -- use PackGrow for grow size

  return bar

-- | Update info status.
statusbarInfoSubitemSetText :: Statusbar -> String -> IO ()
statusbarInfoSubitemSetText = 
    statusbarSubitemSetText . statusbarInfoSubitem

-- | Set text internal.
statusbarSubitemSetText :: StatusbarSubitem -> String -> IO ()
statusbarSubitemSetText ss str = do
  id <- statusbarGetContextId ss str
  statusbarPush ss id str
  return ()

-- | Add new info item to info subitem.
statusbarInfoItemAdd :: Statusbar -> String -> String -> IO ()
statusbarInfoItemAdd = statusbarInfoItemUpdate

-- | Update info item.
statusbarInfoItemUpdate :: Statusbar -> String -> String -> IO ()
statusbarInfoItemUpdate statusbar item = 
    statusbarApplyInfoTable statusbar . M.insert item

-- | Remove info item from info subitem.
statusbarInfoItemRemove :: Statusbar -> String -> IO ()
statusbarInfoItemRemove statusbar = 
    statusbarApplyInfoTable statusbar . M.delete

-- | Apply info table.
statusbarApplyInfoTable :: Statusbar -> (InfoTable -> InfoTable) -> IO ()
statusbarApplyInfoTable statusbar f = do
  -- Apply function.
  modifyIORef (statusbarInfoTable statusbar) f

  -- Update info subitem.
  statusbarInfoSubitemUpdate statusbar

-- | Update info item subitem information.
statusbarInfoSubitemUpdate :: Statusbar -> IO ()
statusbarInfoSubitemUpdate statusbar = do
  infoList <- (<<<=) M.toList (readIORef . statusbarInfoTable) statusbar 
  unlessNull infoList (statusbarInfoSubitemSetText statusbar (concatMap (\x -> snd x ++ " ") infoList))
