module Manatee.Widget.Tooltip where

import Control.Monad
import Control.Monad.Trans

import Graphics.UI.Gtk 
import Graphics.UI.Gtk.Gdk.EventM

data Tooltip =
    Tooltip {tooltipWindow           :: Window -- window for contain widget
            ,tooltipDependWindow     :: Window -- depend window for control display
            ,tooltipLabel            :: Label  -- label to container text
            ,tooltipX                :: Int    -- X coordinate
            ,tooltipY                :: Int    -- Y coordinate
            ,tooltipTimeout          :: Int    -- timeout for display (in millisecond)
            }

-- | New tooltip.
tooltipNew :: Window -> String -> Int -> Int -> Int -> IO Tooltip
tooltipNew win text x y timeout = do
  -- Create tooltip window.
  window <- windowNewPopup
  windowSetDecorated window False

  -- Set left-up coordinate.
  windowMove window x y
  -- windowSetPosition window WinPosCenterAlways

  -- Add label to window.
  frame <- frameNew
  label <- labelNew $ Just text
  frame `containerAdd` label
  set frame [frameShadowType := ShadowEtchedIn]
  window `containerAdd` frame

  -- Create tooltip.
  let tooltip = Tooltip window win label x y timeout

  -- Show window.
  widgetShowAll window

  -- Destory tooltip when out of timeout.
  timeoutAdd (tooltipExit tooltip >> return False) timeout

  -- Handle event.
  win `on` keyPressEvent $ tryEvent $ do
    liftIO $ tooltipExit tooltip
    stopEvent

  return tooltip

-- | Exit tooltip.
tooltipExit :: Tooltip -> IO ()
tooltipExit = widgetDestroy . tooltipWindow
