{-# LANGUAGE ExistentialQuantification, RankNTypes #-}
{-# LANGUAGE DeriveDataTypeable, TemplateHaskell #-}
module Reboot where

import Data.Binary
import Data.IORef
import Core
import Data.DeriveTH

import Manatee.IORefObject
import Manatee.State.Page
import Manatee.State.ViewNode
import Manatee.State.View

import qualified Data.Set as Set

data RebootState = 
    RebootState {rebootViewNodeState    :: ViewNodeListState
                ,rebootPageState        :: PageListState
                ,rebootViewState        :: ViewListState
                ,rebootTerminalCounter  :: Int}

-- | Reboot.
reboot :: IORefObject -> IO ()
reboot ioRefObject = 
    relaunch =<< rebootGetState ioRefObject

-- | Default state for reboot.
rebootDefaultState :: RebootState
rebootDefaultState = 
    RebootState (ViewNodeListState Set.empty 0) (PageListState Set.empty 0 Nothing) (ViewListState Nothing []) 0

-- | Get reboot state.
rebootGetState :: IORefObject -> IO RebootState
rebootGetState ioRefObject = do
  -- Get view node state.
  vnss <- viewNodeListStateGet ioRefObject
  -- Get page state.
  pss  <- pageListStateGet ioRefObject
  -- Get view state.
  vls  <- viewListStateGet ioRefObject
  -- Get terminal counter.
  cnt <- readIORef $ ioRefObjectTerminalCounter ioRefObject

  return $ RebootState vnss pss vls cnt

-- | Rebuild reboot state.
rebootRebuild :: RebootState -> IORefObject -> IO ()
rebootRebuild rs ioRefObject = do
  -- Rebuild view node state.
  viewNodeListStateRebuild (rebootViewNodeState rs) ioRefObject
  -- Rebuild page state.
  pageListStateRebuild (rebootPageState rs) ioRefObject
  -- Rebuild view state.
  viewListStateRebuild (rebootViewState rs) ioRefObject
  -- Rebuild terminal counter.
  modifyIORef (ioRefObjectTerminalCounter ioRefObject) (const $ rebootTerminalCounter rs) 

$(derive makeBinary ''RebootState)
